//
//  BMKGeofenceController.m
//  BMKGeofenceOCDemo
//
//  Created by v_chiyuanwei on 2020/7/15.
//  Copyright © 2020 baidu. All rights reserved.
//

#import "BMKGeofenceController.h"

#import <BaiduMapAPI_Map/BMKMapComponent.h>
#import <BMKLocationkit/BMKLocationComponent.h>

#import "BMKGeofenceTabView.h"
#import "BMKGeofenceMenuView.h"

@interface BMKGeofenceController () <BMKMapViewDelegate, BMKLocationManagerDelegate, BMKGeoFenceManagerDelegate>
/// 地图
@property (nonatomic, strong) BMKMapView *mapView;
/// 地理围栏管理类
@property (nonatomic, strong) BMKGeoFenceManager *geofenceManager;
/// 定位管理类
@property (nonatomic, strong) BMKLocationManager *locationManager;
/// 添加、删除围栏
@property (nonatomic, strong) BMKGeofenceTabView *tabView;
/// 围栏参数设置
@property (nonatomic, strong) BMKGeofenceMenuView *menuView;
/// HUD
@property (nonatomic, strong) UILabel *alertLabel;

@end

@implementation BMKGeofenceController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = @"地理围栏";
    
    [self initialize];
    [self setupViews];
    
    // 开始连续定位。
    [self.locationManager startUpdatingLocation];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    // 当mapView即将被显示的时候调用，恢复之前存储的mapView状态
    [self.mapView viewWillAppear];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // 当mapView即将被隐藏的时候调用，存储当前mapView的状态
    [self.mapView viewWillDisappear];
}

#pragma mark - UI
- (void)initialize {
    self.view.backgroundColor = [UIColor whiteColor];
    self.navigationController.navigationBar.translucent = NO;
    self.navigationController.navigationBar.barStyle = UIBarStyleBlack;
    self.navigationController.navigationBar.barTintColor = [UIColor colorWithRed:34.f / 255.f green:37.f / 255.f blue:61.f / 255.f alpha:1.f];
    [self.navigationController.navigationBar setTitleTextAttributes:@{NSForegroundColorAttributeName:[UIColor whiteColor]}];
}

- (void)setupViews {
    [self.view addSubview:self.mapView];
    [self.view addSubview:self.menuView];
    [self.view addSubview:self.tabView];
    [self.view addSubview:self.alertLabel];
}

#pragma mark - 围栏相关操作
/// 添加圆形围栏
- (void)addCircleFence {
    CGFloat fenceRadius = self.menuView.fenceRadius;
    CLLocationCoordinate2D centerCoord = self.menuView.fenceCenterCoord;
    
    if (fenceRadius <= 0) {
        [self showMsg:@"请设置围栏半径"];
        return;
    }
    
    if (centerCoord.latitude == 0 && centerCoord.longitude == 0) {
        [self showMsg:@"请设置围栏中心点"];
        return;
    }

    // 添加圆形overlay
    BMKCircle *circleOverlay = [BMKCircle circleWithCenterCoordinate:centerCoord radius:fenceRadius];
    [self.mapView addOverlay:circleOverlay];
    // 添加圆形围栏
    [self.geofenceManager addCircleRegionForMonitoringWithCenter:centerCoord radius:fenceRadius coorType:BMKLocationCoordinateTypeBMK09LL customID:self.menuView.fenceID];
}

/// 添加多边形围栏
- (void)addPolygonFence {
    
    if (self.menuView.polygonFenceCoords.count < 3) {
        [self showMsg:@"c"];
        return;
    }
    
    NSInteger count = self.menuView.polygonFenceCoords.count;
    CLLocationCoordinate2D coords[count];
    
    for (int i = 0; i < count; i++) {
        NSValue *value = self.menuView.polygonFenceCoords[i];
        CLLocationCoordinate2D coord;
        [value getValue:&coord];
        coords[i] = coord;
    }

    // 添加多边形overlay
    BMKPolygon *polygon = [BMKPolygon polygonWithCoordinates:coords count:count];
    [self.mapView addOverlay:polygon];
    // 添加多边形围栏
    [self.geofenceManager addPolygonRegionForMonitoringWithCoordinates:coords count:count coorType:BMKLocationCoordinateTypeBMK09LL customID:self.menuView.fenceID];
}

/// 移除所有围栏
- (void)romeveAllFence {
    // 移除所有annotations
    [self.mapView removeAnnotations:self.mapView.annotations];
    // 移除所有overlays
    [self.mapView removeOverlays:self.mapView.overlays];
    // 移除所有围栏
    [self.geofenceManager removeAllGeoFenceRegions];
    // 停止连续定位。
    [self.locationManager stopUpdatingLocation];
    
    self.menuView.fenceCenterCoord = CLLocationCoordinate2DMake(0, 0);
    [self.menuView.polygonFenceCoords removeAllObjects];
    
    if (self.menuView.fenceType == 1) {
        self.menuView.coordsInfoStr = @"请点击地图选择地图中心点";
    } else {
        self.menuView.coordsInfoStr = @"请点击地图选择围栏的边界点，至少3个";
    }
}

#pragma mark - 菜单
/// 显示菜单
- (void)showMenu {
    self.menuView.hidden = NO;
    [UIView animateWithDuration:0.2f animations:^{
        CGRect frame = self.menuView.frame;
        frame.origin.y = 60;
        self.menuView.frame = frame;
    }];
}

/// 隐藏菜单
- (void)hideMenu {
    [UIView animateWithDuration:0.2 animations:^{
        CGRect frame = self.menuView.frame;
        frame.origin.y = -150;
        self.menuView.frame = frame;
    } completion:^(BOOL finished) {
        self.menuView.hidden = YES;
    }];
}

/// alert msg
- (void)showMsg:(NSString *)msg {
    if (msg.length == 0 || !self.alertLabel.isHidden) return;
    
    CGSize msgSize = [msg boundingRectWithSize:CGSizeMake(self.view.frame.size.width - 40, 40) options:NSStringDrawingUsesFontLeading attributes:@{NSFontAttributeName: [UIFont systemFontOfSize:14]} context:nil].size;
    self.alertLabel.frame = CGRectMake((SCREEN_WIDTH - msgSize.width - 20) / 2, CONTENT_HEIGHT / 2, msgSize.width + 20, msgSize.height + 16);
    self.alertLabel.text = msg;
    self.alertLabel.hidden = NO;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        self.alertLabel.hidden = YES;
    });
}

#pragma mark - <BMKMapViewDelegate>
/// 根据overlay生成对应的View
- (BMKOverlayView *)mapView:(BMKMapView *)mapView viewForOverlay:(id<BMKOverlay>)overlay {
    if ([overlay isKindOfClass:[BMKCircle class]]) { /// 圆
        BMKCircleView *circleView = [[BMKCircleView alloc] initWithCircle:overlay];
        circleView.fillColor = [[UIColor alloc] initWithRed:0.0f green:0.0f blue:1.0f alpha:0.1f];
        circleView.strokeColor = [[UIColor alloc] initWithRed:0.0f green:0.0f blue:1.0f alpha:0.3f];
        circleView.lineWidth = 1.0f;
        
        return circleView;
    } else if ([overlay isKindOfClass:[BMKPolygon class]]) { /// 多边形
        BMKPolygonView *polygonView = [[BMKPolygonView alloc] initWithPolygon:overlay];
        polygonView.fillColor = [UIColor colorWithRed:0.0f green:0.0f blue:1.0f alpha:0.1f];
        polygonView.strokeColor = [UIColor colorWithRed:0.0f green:0.0f blue:1.0f alpha:0.3f];
        polygonView.lineWidth = 1.0f;
        
        return polygonView;
    }
   
    return nil;
}

/// 点中底图空白处会回调此接口
- (void)mapView:(BMKMapView *)mapView onClickedMapBlank:(CLLocationCoordinate2D)coordinate {
    if (self.menuView.fenceType == 1) { // 圆形围栏
        // 点击地图设置地理围栏中心
        self.menuView.fenceCenterCoord = coordinate;
        [mapView removeAnnotations:mapView.annotations];
    } else { // 多边形边界点
        // 保存多边形边界点
        NSValue *coordValue = [NSValue valueWithBytes:&coordinate objCType:@encode(struct CLLocationCoordinate2D)];
        [self.menuView.polygonFenceCoords addObject:coordValue];
        self.menuView.coordsInfoStr = [NSString stringWithFormat:@"已选择%zd个点",self.menuView.polygonFenceCoords.count];
    }
    // 添加annotation
    BMKPointAnnotation *annotation = [[BMKPointAnnotation alloc] init];
    annotation.coordinate = coordinate;
    [mapView addAnnotation:annotation];
    
    [self.menuView endEditing:YES];
}

#pragma mark - <BMKGeoFenceManagerDelegate>
/// 后台定位权限
- (void)BMKGeoFenceManager:(BMKGeoFenceManager * _Nonnull)manager doRequestAlwaysAuthorization:(CLLocationManager * _Nonnull)locationManager {
    [locationManager requestAlwaysAuthorization];
}

/// 添加地理围栏完成后的回调，成功与失败都会调用
- (void)BMKGeoFenceManager:(BMKGeoFenceManager * _Nonnull)manager didAddRegionForMonitoringFinished:(NSArray <BMKGeoFenceRegion *> * _Nullable)regions customID:(NSString * _Nullable)customID error:(NSError * _Nullable)error {
    if (error) {
        NSLog(@"geofence error = %@", error);
    } else {
        NSLog(@"add geofence %@ success", regions.firstObject.customID);
        
        // 添加成功移除上一次的数据
        if (self.menuView.fenceType == 1) {
            self.menuView.fenceCenterCoord = CLLocationCoordinate2DMake(0, 0);
            self.menuView.coordsInfoStr = @"请点击地图选择地图中心点";
        } else {
            [self.menuView.polygonFenceCoords removeAllObjects];
            [self.mapView removeAnnotations:self.mapView.annotations];
            self.menuView.coordsInfoStr = @"请点击地图选择围栏的边界点，至少3个";
        }
    }
}

/// 地理围栏状态改变时回调，当围栏状态的值发生改变，定位失败都会调用
- (void)BMKGeoFenceManager:(BMKGeoFenceManager * _Nonnull)manager didGeoFencesStatusChangedForRegion:(BMKGeoFenceRegion * _Nullable)region customID:(NSString * _Nullable)customID error:(NSError * _Nullable)error {
    if (error) {
        NSLog(@"geofence error = %@", error);
        return;
    }
    
    NSLog(@"geofence %@ status = %zd", region.customID, region.fenceStatus);
    switch(region.fenceStatus) {
        case BMKGeoFenceRegionStatusInside:
            [self showMsg:[NSString stringWithFormat:@"进入地理围栏 业务ID：%@ 地理围栏ID：%@", region.customID, region.identifier]];
            break;
        case BMKGeoFenceRegionStatusStayed:
            [self showMsg:[NSString stringWithFormat:@"停留在地理围栏 业务ID：%@ 地理围栏ID：%@", region.customID, region.identifier]];
            break;
        case BMKGeoFenceRegionStatusOutside:
            [self showMsg:[NSString stringWithFormat:@"离开地理围栏 业务ID：%@ 地理围栏ID：%@", region.customID, region.identifier]];
            break;
        default:
            [self showMsg:[NSString stringWithFormat:@"围栏状态未知 业务ID：%@ 地理围栏ID：%@", region.customID, region.identifier]];
            break;
    }
}

#pragma mark - <BMKLocationManagerDelegate>
///  连续定位回调函数。
- (void)BMKLocationManager:(BMKLocationManager *)manager didUpdateLocation:(BMKLocation *)location orError:(NSError *)error {
    // 定位错误
    if (error) {
        NSLog(@"update Location error: %@", error.localizedDescription);
        return;
    }
    // 定位结果
    if (location) {
        // 显示当前位置
        BMKUserLocation *userLoc = [[BMKUserLocation alloc] init];
        userLoc.location = location.location;
        [self.mapView updateLocationData:userLoc];
        // 设置地图中心
        self.mapView.centerCoordinate = location.location.coordinate;
    }
}

#pragma mark - getter
/// 地图
- (BMKMapView *)mapView {
    if (!_mapView) {
        _mapView = [[BMKMapView alloc] initWithFrame: CGRectMake(0, 0, SCREEN_WIDTH, CONTENT_HEIGHT)];
        // 设定是否显示定位图层
        _mapView.showsUserLocation = YES;
        // 地图比例尺级别
        _mapView.zoomLevel = 17;
        // 设置delegate
        _mapView.delegate = self;
    }
    return _mapView;
}

/// 地理围栏管理类
- (BMKGeoFenceManager *)geofenceManager {
    if (!_geofenceManager) {
        _geofenceManager = [[BMKGeoFenceManager alloc] init];
        /// 指定定位是否会被系统自动暂停。默认为NO。
        _geofenceManager.pausesLocationUpdatesAutomatically = NO;
        /// 是否允许后台定位。
        _geofenceManager.allowsBackgroundLocationUpdates = YES;
        /// 需要进行通知的行为 
        _geofenceManager.activeAction = BMKGeoFenceActiveActionInside;
        /// 代理
        _geofenceManager.delegate = self;
    }
    return _geofenceManager;
}

/// 定位管理类
- (BMKLocationManager *)locationManager {
    if (!_locationManager) {
        _locationManager = [[BMKLocationManager alloc] init];
        // 设置返回位置的坐标系类型
        _locationManager.coordinateType = BMKLocationCoordinateTypeBMK09LL;
        // 设置距离过滤参数
        _locationManager.distanceFilter = 5.0;
        // 设置预期精度参数
        _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
        // 设置应用位置类型
        _locationManager.activityType = CLActivityTypeAutomotiveNavigation;
        // 设置是否自动停止位置更新
        _locationManager.pausesLocationUpdatesAutomatically = NO;
        // 允许后台定位
        _locationManager.allowsBackgroundLocationUpdates = YES;
        // 设置位置获取超时时间
        _locationManager.locationTimeout = 10;
        // 设置获取地址信息超时时间
        _locationManager.reGeocodeTimeout = 10;
        // 设置delegate
        _locationManager.delegate = self;
    }
    return _locationManager;
}

/// 添加、删除围栏
- (BMKGeofenceTabView *)tabView {
    if (!_tabView) {
        _tabView = [[BMKGeofenceTabView alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, 60)];
        
        __weak typeof(self) weakSelf = self;
        _tabView.didClickTabBlock = ^(NSInteger idx, BOOL isSelected) {
            if (idx == 0) { //移除所有围栏
                [weakSelf romeveAllFence];
            } else if (idx == 1) {
                weakSelf.geofenceManager.activeAction = weakSelf.menuView.activeAction;
                if (self.menuView.fenceType == 1) {
                    [weakSelf addCircleFence]; // 添加圆形围栏
                } else {
                    [weakSelf addPolygonFence]; // 添加多边形围栏
                }
            } else if (idx == 2) {
                if (isSelected) {
                    [weakSelf showMenu]; // 显示菜单
                } else {
                    [weakSelf hideMenu]; // 隐藏菜单
                    [weakSelf.menuView endEditing:YES];
                }
            }
        };
    }
    return _tabView;
}

/// 围栏参数设置view
- (BMKGeofenceMenuView *)menuView {
    if (!_menuView) {
        _menuView = [[BMKGeofenceMenuView alloc] initWithFrame:CGRectMake(0, 60, SCREEN_WIDTH, 150)];
        
        __weak typeof(self) weakSelf = self;
        _menuView.fenceTypeChangedBlock = ^(NSInteger fenceType) {
            if (fenceType == 1) {
                [weakSelf.mapView removeAnnotations:weakSelf.mapView.annotations];
            }
        };
    }
    return _menuView;
}

/// HUD
- (UILabel *)alertLabel {
    if (!_alertLabel) {
        _alertLabel = [[UILabel alloc] init];
        _alertLabel.backgroundColor = [UIColor colorWithWhite:0.0f alpha:0.5];
        _alertLabel.font = [UIFont systemFontOfSize:14];
        _alertLabel.textColor = [UIColor whiteColor];
        _alertLabel.textAlignment = NSTextAlignmentCenter;
        _alertLabel.layer.cornerRadius = 8;
        _alertLabel.layer.masksToBounds = YES;
        _alertLabel.hidden = YES;
    }
    return _alertLabel;
}

@end
